import org.scalatest.FunSuite 
import org.scalactic.Tolerance

class CalculatorCheckSuite extends FunSuite with Tolerance {

  def checkExpr(expr: String, result: Double) {
    val tok = new Tokenizer(expr)
    val t = Calculator4.parseExpression(tok)
    if (!tok.token.isStop)
      fail("Could not parse expression '%s'".format(expr))
    assert(t === (result +- 1e-13))
  }

  def checkError(expr: String, pos: Int, msg: String) {
    val tok = new Tokenizer(expr)
    try {
      val t = Calculator4.parseExpression(tok)
      if (!tok.token.isStop)
	fail("Could not parse expression '%s'".format(expr))
      fail("No error was reported in expression '%s'".format(expr))
    } catch {
      case e: SyntaxError => 
	assert(pos == e.pos && msg == e.msg)
    }
  }

  test("Constants") {
    checkExpr("pi", math.Pi)
    checkExpr("2 * pi", 2 * math.Pi)
    checkExpr("e", math.E)
    checkExpr("e^2", math.E * math.E)
    checkExpr("1 - e^-1", 1.0 - 1/math.E)
  }

  test("Division by zero") {
    checkError("4.5 / 0", 4, "Division by zero")
    checkError("1 + 2 + 3 + 4 / (10 - 2 * 5)", 14, "Division by zero")
  }

  test("Functions") {
    checkExpr("sqrt(16)", 4.0)
    checkExpr("3 + sqrt(81)", 12.0)
    checkExpr("sin(pi/3)", math.sin(math.Pi/3.0))
    checkExpr("1/sqrt(2)", 1.0 / math.sqrt(2.0))
    checkExpr("1/sqrt(3)", 1.0 / math.sqrt(3.0))
    checkExpr("cos(pi/3)", math.cos(math.Pi/3.0))
  }

  test("Function syntax") {
    checkError("sqrt + 7", 5, "expected '('")
    checkError("sqrt 2", 5, "expected '('")
    checkError("sin pi", 4, "expected '('")
    checkError("sin - pi", 4, "expected '('")
    checkError("7 / pi + 3 * sqrt", 17, "expected '('")
    checkError("sqrt(13 + 9", 11, "expected operator or ')'")
  }

  test("Sqrt argument") {
    checkError("sqrt(-7)", 0, "Square root of negative number")
    checkError("3 + sqrt(22 - 3 * 13)", 4, "Square root of negative number")
  }
}
  
